<?php
/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://wpruby.com
 * @since      1.0.0
 *
 * @package    Restricted_Shipping_And_Payment_For_Woocommerce
 * @subpackage Restricted_Shipping_And_Payment_For_Woocommerce/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Restricted_Shipping_And_Payment_For_Woocommerce
 * @subpackage Restricted_Shipping_And_Payment_For_Woocommerce/includes
 * @author     WPRuby <info@wpruby.com>
 */
class RSPW {



	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      RSPW_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		if ( defined( 'RSPW_VERSION' ) ) {
			$this->version = RSPW_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		$this->plugin_name = 'restricted-shipping-and-payment-for-woocommerce';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();
	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - RSPW_Loader. Orchestrates the hooks of the plugin.
	 * - RSPW_I18n. Defines internationalization functionality.
	 * - RSPW_Admin. Defines all hooks for the admin area.
	 * - RSPW_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-rspw-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-rspw-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-rspw-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-rspw-public.php';

		/**
		 * Loading CMB2 library
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/vendors/cmb2/init.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/vendors/cmb-field-select2/cmb-field-select2.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/vendors/cmb-field-time-range/cmb-field-time-range.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/abstract-rspw-filter.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-rspw-shipping-filter.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-rspw-payment-filter.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-rspw-meta-box.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-rspw-shipping-conditions-meta-box.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-rspw-payment-conditions-meta-box.php';

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-rspw-plugin-updater.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-rspw-license-handler.php';

		$this->loader = new RSPW_Loader();
	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the RSPW_I18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {
		$plugin_i18n = new RSPW_I18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );
	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {
		$plugin_admin = new RSPW_Admin( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );
		$this->loader->add_action( 'init', $plugin_admin, 'register_post_types' );
		$this->loader->add_action( 'wp_ajax_get_shipping_rule_type_operators', $plugin_admin, 'get_shipping_rule_type_operators' );
		$this->loader->add_action( 'wp_ajax_get_payment_rule_type_operators', $plugin_admin, 'get_payment_rule_type_operators' );
	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {
		$plugin_public = new RSPW_Public( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );
	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    RSPW_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

    public static function get_shipping_methods() {
        $methods = array();
        $zones   = WC_Shipping_Zones::get_zones();

        if ( ! isset( $zones[0] ) ) {
            $rest_of_world                = WC_Shipping_Zones::get_zone_by();
            $zones[0]                     = $rest_of_world->get_data();
            $zones[0]['shipping_methods'] = $rest_of_world->get_shipping_methods();
        }

        foreach ( $zones as $zone ) {
            if ( ! empty( $zone['shipping_methods'] ) ) {
                $zone_name = $zone['zone_name'];

                foreach ( $zone['shipping_methods'] as $instance_id => $method_instance ) {
                    $option_id             = $method_instance->get_rate_id();
                    $method_title          = sprintf( __( '&quot;%1$s&quot; (Instance ID: %2$s)', 'restricted-shipping-and-payment-for-woocommerce' ), $method_instance->get_title(), $instance_id );
                    $option_name           = sprintf( __( '%1$s &ndash; %2$s', 'restricted-shipping-and-payment-for-woocommerce' ), $zone_name, $method_title );
                    $methods[ $option_id ] = $option_name;
                }
            }
        }

        return $methods;
    }
}
